#!/bin/awk -f
#
#       %Z% %M% %I% %E% %Q%
#
#******************************************************************************
#       Copyright (C) 2000 The Santa Cruz Operation, Inc.
#               All Rights Reserved.
#
#       The information in this file is provided for the exclusive use of
#       the licensees of The Santa Cruz Operation, Inc.  Such users have the
#       right to use, modify, and incorporate this code into other products
#       for purposes authorized by the license agreement provided they include
#       this notice and the associated copyright notice with any such product.
#       The information in this file is provided "AS IS" without warranty.
#==============================================================================
#
# Upgrade.awk translates an old  Squid configuration file to the current format.
# This will retain/merge user's existing options from the older file
# but will lose their comments.
# Syntax: upgrade.awk convert_file=<fileToConvert> <templateFile>
# fileToConvert is the old file.
# templateFile is a current squid.conf file.
# fileToConvert is merged into templateFile
#-------------------------------------------------------------------
# Function: convert
# Main conversion routine
# At return, the global variable new_values will contain the converted
# configuration lines 
# @param in_old_values contains an array of configuration lines
# All other parameters are local variables
#-------------------------------------------------------------------
function convert(in_old_values, local_deprecated, i, local_current, local_tag){
    # create a set of deprecated tags
    # These will be used to ensure that no deprecated tags make it into
    # the file that we're creating.  It does not convert them -- that
    # is done below in the main conversion loop.
    local_deprecated["forward_snmpd_port"] = ""
    local_deprecated["request_size"] = ""
    local_deprecated["tcp_incoming_address"] = ""

    # Do a straight copy of old into new, filtering out deprecated tags
    i = 0
    while (i in in_old_values){
 	split(in_old_values[i], local_current)
	local_tag = tolower(local_current[1])
	if ( !(local_tag in local_deprecated) ){
	    add_to_new_values(in_old_values[i])
	    # Build up a set of tags we've seen from the old file.
	    # We'll comment out all such lines in the template file since
	    # they'll already be in the converted section
	    # No need to add deprecated tags -- they won't be in the template
	    converted_tags[local_tag] = ""
	}
	i++
    }

    # Now make tag-specific changes
    # This is the main conversion loop.
    i = 0
    while (i in in_old_values){
	split(in_old_values[i], local_current)
	local_tag = tolower(local_current[1])

	if (local_tag == "tcp_incoming_address"){
	    # local_current[2] is the ip address to listen on
	    convert_tcp_incoming_address(local_current[2])
	}
	else if (local_tag == "forward_snmpd_port"){
	    # can't do anything but explain that it has been deprecated
	    add_to_new_values("# FUNCTIONALITY REMOVED: " in_old_values[i])
	}
	else if (local_tag == "request_size"){
	    converted_tags[local_current[1]] = ""
	    # local_current[2] is request size, local_current[3] is optional units
	    convert_request_size(local_current[2], local_current[3])
	}
	else if (local_tag == "cache_dir"){
	    convert_cache_dir(local_current)
	}
	i++
    }
    
}
#-------------------------------------------------------------------
# Function convert_cache_dir
# Old cache_dir looked like: "cache_dir dirname 100 16 256"
# New cache_dir includes filesys: "cache_dir ufs dirname 100 16 256"
# @param cache_line contains complete line from configuration file
# All other parameters are local variables
#-------------------------------------------------------------------
function convert_cache_dir(config_line, i){
    # if the line has current format, this function should be a no-op.  
    # this can be easily detected by whether or not it has 6 fields
    if (6 in config_line){
	return
    }
    # otherwise, we have to change it -- find the copy in new_values and 
    # change that one
    i = find_in_new_values("cache_dir")
    # i is now pointing to the cache_dir line
    sub(/cache_dir/, "cache_dir ufs ", new_values[i])
}

#-------------------------------------------------------------------
# Function: convert_request_size
# The tag request_size is deprecated, beginning with 2.3.STABLE2.  
# The functionality has been moved into the tags
# request_header_max_size and request_body_max_size.  For the purpose
# of conversion, this function just maps request_size to request_body_max_size
# @param size is a numeric size, 
# @param units is an element from {Bytes, KB, MB, GB} or blank
# All other parameters are local variables
#-------------------------------------------------------------------
function convert_request_size(size, units, local_current, i){
    i = find_in_new_values("request_body_max_size")
    if (i != -1){
	split(new_values[i], local_current)
	# They've already set the new values -- just remove request_size
	add_to_new_values("#REPLACED by request_header_max_size: request_size " size " " units)
    }
    else {
	add_to_new_values("# Following request_body_max_size replaces deprecated request_size tag\nrequest_body_max_size " size " " units)
    }
}

#-------------------------------------------------------------------
# Function: convert_tcp_incoming_address
# The tag tcp_incoming_address is deprecated, beginning with 2.3.STABLE2.
# The functionality has been moved into the tag http_port
# @param ip_address contains an ip address to listen on
# All other parameters are local variables
#-------------------------------------------------------------------
function convert_tcp_incoming_address(ip_address, local_current, i){
    i = find_in_new_values("http_port")
    if (i != -1){
	split(new_values[i], local_current)
	# configuration line currently looks like http_port port1 port2 ....
	# after modification, it should look like http_port ip:port1 ip:port2 ....
	new_values[i] = "# Following http_port line contains merged information from\n"
	new_values[i] = new_values[i] "# deprecated tcp_incoming_address tag\n" 
	new_values[i] = new_values[i] local_current[1]
	for (j = 2; local_current[j] != ""; j++){
	    new_values[i] = new_values[i] " " ip_address ":" local_current[j]
	}
    }
    else {
	add_to_new_values("# Following http_port line replaces deprecated tcp_incoming_address tag\n" "http_port " ip_address)
    }
}

#-------------------------------------------------------------------
# Function add_to_new_values provides an api for adding a new line
# to the new_values array.
# @param line is the line to add
#-------------------------------------------------------------------
function add_to_new_values(line){
    new_values[num_new_values++] = line
}

#-------------------------------------------------------------------
# Function: find_in_new_values
# returns the index of the first configuration line in new_values 
# that matches the given tag.  Returns -1 if tag is not found
# @param tag is the tag to search for, all other parameters are
# local variables
#-------------------------------------------------------------------
function find_in_new_values(tag, local_current, i){
    i = 0
    while (i in new_values){
	split(new_values[i], local_current)
	if ( tolower(local_current[1]) == tag ){
	    return i
	}
	i++
    }
    return (i in new_values) ? i : -1
}

#-------------------------------------------------------------------
# Function: print_new_values
# prints out the converted configuration
#-------------------------------------------------------------------
function print_new_values(){
    print "#################################################################"
    print "# Following section was converted from " convert_file
    print "#################################################################"
    print ""
    for (i = 0; i < num_new_values; i++){
	print new_values[i]
    }
    print ""
    print "#################################################################"
    print "# End conversion of " convert_file
    print "#################################################################"
    print ""
}

BEGIN {
    # First file is old-format.  Keep all values it sets so we can look up old
    # values later as we need them.  Can't hash on tag, because there are some
    # that will be seen multiple times.  For example:
    # http_access deny foo
    # http_access deny bar
    # http_access allow all
    # is perfectly legal.  For simplicity, just keep all and iterate through
    # the list later when we need them.  
    
    while ( (getline < convert_file) > 0 ){
	# Squid configuration file is made up of lines containing 'tag value'
	# So ignore lines that don't have at least 2 fields.  Also ignore 
	# lines starting with the comment character '#'
 	if ( ($0 !~ /^#/) && (NF > 1) ){
	    old_values[i++] = $0
	}
    }

    convert(old_values)
    print_new_values()

}

# tags are added to converted_tags when they are converted from the old-format
# file.  All such tags are already at the top of the file.  Hence, we should
# comment out any such tags we see in the template file.
$1 in converted_tags {
    print "# Next line commented out since a version from " convert_file " is at the top of this file"
    print "#" $0
    next
}

# Otherwise, print it out
{ print $0 }
